# ----------------------------------------------------------------------------
#
#  Welcome to Baml! To use this generated code, please run the following:
#
#  $ pip install baml
#
# ----------------------------------------------------------------------------

# This file was generated by BAML: please do not edit it. Instead, edit the
# BAML files and re-generate this code using: baml-cli generate
# baml-cli is available with the baml package.

from . import types
from . import stream_types


type_map = {

    "types.Feedback": types.Feedback,
    "stream_types.Feedback": stream_types.Feedback,

    "types.History": types.History,
    "stream_types.History": stream_types.History,

    "types.Output": types.Output,
    "stream_types.Output": stream_types.Output,

    "types.PageAbstract": types.PageAbstract,
    "stream_types.PageAbstract": stream_types.PageAbstract,

    "types.Step": types.Step,
    "stream_types.Step": stream_types.Step,

    "types.TestCase": types.TestCase,
    "stream_types.TestCase": stream_types.TestCase,


}